/* This file is part of kmobiletools.
   See COPYING file for licensing information.

   (C) 2005 Marco Gulino <marco.gulino@gmail.com>
   (C) 2005 Stefan Bogner <bochi@kmobiletools.org>

*/


#ifndef GAMMUJOBS_H
#define GAMMUJOBS_H

#include "kmobiletoolsengine.h"
#include "kmobiletoolsgammu_engine.h"
#include "qphoneerror.h"

#include <qptrlist.h>
#include <kabc/addressee.h>
#include <qptrvector.h>

extern "C" {
    #include <gammu/gammu.h>
}
#undef bool

class Device;
class QStringList;
class kmobiletoolsGammu_engine;

class kmobiletoolsGammuJob : public kmobiletoolsJob
{
    public:
        kmobiletoolsGammuJob(Device *device, kmobiletoolsGammu_engine* parent = 0 , const char* name = 0) : kmobiletoolsJob( (QObject*)parent,name)
        { p_device=device; engine = parent; }

    protected:
        Device *p_device;
        kmobiletoolsGammu_engine *engine;
        GSM_Error error;
        int i_retry;
};

class FetchPhoneInfos : public kmobiletoolsGammuJob
{
    public:
        FetchPhoneInfos (Device *dev, kmobiletoolsGammu_engine* parent = 0 , const char* name = 0);
        int type()            { return kmobiletoolsJob::fetchPhoneInfos; }
        QString model() const { return s_model; }
        QString manufacturer() const { return s_manufacturer; }
        QString revision() const { return s_firmware; }
        QString imei() const { return s_imei; }

    protected:
        void run () ;

    private:
        Device *dev;
        QString s_model, s_manufacturer, s_firmware, s_imei;
};

class PollStatus : public kmobiletoolsGammuJob
{
    public:
        PollStatus (Device *dev, kmobiletoolsGammu_engine* parent = 0 , const char* name = 0);
        int phoneCharge()     { return s_battery; }
        int phoneSignal()     { return s_signal; }
        QString networkName() { return s_network; }
        int unreadSMS()       { return s_unreadSMS; }
        int totalSMS()        { return s_totalSMS; }
        bool ringing()        { return s_ringing; }
        int type()            { return kmobiletoolsJob::pollStatus; }
    protected:
        void run ();
    private:
        int s_battery, s_signal, s_unreadSMS, s_totalSMS;
        QString s_network;
        bool s_ringing;
        QPhoneError err;
};


#endif
