/* This file is part of kmobiletools.
   See COPYING file for licensing information.

   (C) 2005 Marco Gulino <marco.gulino@gmail.com>
   (C) 2005 Stefan Bogner <bochi@kmobiletools.org>

*/

#include "memory_jobs.h"

#include "qregexp.h"
#include <unistd.h>
#include <qstringlist.h>
#include <qdatetime.h>
#include <kabc/vcardconverter.h>
#include "kmobiletoolsgammu_engine.h"
#include <kdebug.h>


FetchMemory::FetchMemory( Device *device, kmobiletoolsGammu_engine* parent, const char* name ) : kmobiletoolsGammuJob(device, parent, name) 
{}

void FetchMemory::run()
{
    getPbk( MEM_ME );
    getPbk( MEM_SM );
}

void FetchMemory::getPbk( GSM_MemoryType type )
{
    int x = 0;
    m.MemoryType = type;
    error = p_device->phone->GetMemoryStatus(&p_device->s, &m);
    int mSize = m.MemoryUsed + m.MemoryFree;
    kdDebug() << "[ !!! ] Fetching the Phonebook for Memorytype " << m.MemoryType << ", Result: " << m.MemoryUsed << " entries, mSize is " << m.MemoryUsed << ", " << err.getErrorMsg(error) << endl;

    for( int i=1; i < mSize; i++ ) {

        Entry.MemoryType = m.MemoryType;
        Entry.Location = i;

        error = p_device->phone->GetMemory(&p_device->s, &Entry);
        kdDebug() << "[ !!! ] " << err.getErrorMsg(error) << endl;

        if ( error == ERR_EMPTY ) freeEntries.append( Entry );
        else 
        {
            addresseeList.append( pbk2kabc( Entry ) );
            x++;
        }

        if( error == ERR_EMPTY && x == m.MemoryUsed && !freeEntries.isEmpty() ) break;

    }
}

KABC::Addressee* FetchMemory::pbk2kabc( GSM_MemoryEntry Entry )
{
    kdDebug() << "[ !!! ] pbk2abc " << Entry.EntriesNum << endl;
    KABC::Addressee *a = new KABC::Addressee();
    KABC::Address address( KABC::Address::Postal );
    int name, number, group;

    GSM_PhonebookFindDefaultNameNumberGroup(&Entry, &name, &number, &group);
    
    if( Entry.EntriesNum == 1 ) 
    {
        a->setFormattedName( (char*)DecodeUnicodeConsole( Entry.Entries[name].Text ) ); 
        a->setFamilyName( (char*)DecodeUnicodeConsole( Entry.Entries[name].Text ) );
        a->insertPhoneNumber( KABC::PhoneNumber( (char*)DecodeUnicodeConsole( Entry.Entries[number].Text ), KABC::PhoneNumber::Pref ) );
    }
    //a->setFormattedName( (char*)DecodeUnicodeConsole( Entry.Entries[name].Text ) );
    //a->setFamilyName( (char*)DecodeUnicodeConsole( Entry.Entries[name].Text ) );
    //a->insertPhoneNumber( KABC::PhoneNumber( (char*)DecodeUnicodeConsole( Entry.Entries[number].Text ), KABC::PhoneNumber::Pref ) );
    a->insertCustom("KMobileTools", "index", QString::number( Entry.Location ) );
    a->insertCustom("KMobileTools", "memslot", QString::number( Entry.MemoryType ) );

    for (int x=0;x<Entry.EntriesNum;x++) {

        switch (Entry.Entries[x].EntryType) {

            case PBK_Category:
                a->insertCategory( (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_Private:
                a->insertCustom("KMobileTools", "private", (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_Number_General:
                a->insertPhoneNumber( KABC::PhoneNumber( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ), KABC::PhoneNumber::Home ) );
            break;

            case PBK_Number_Mobile:
                a->insertPhoneNumber( KABC::PhoneNumber( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ), KABC::PhoneNumber::Cell ) );
            break;

            case PBK_Number_Work:
                a->insertPhoneNumber( KABC::PhoneNumber( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ), KABC::PhoneNumber::Work ) );
            break;

            case PBK_Number_Fax:
                a->insertPhoneNumber( KABC::PhoneNumber( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ), KABC::PhoneNumber::Fax ) );
            break;

            case PBK_Number_Home:
                a->insertPhoneNumber( KABC::PhoneNumber( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ), KABC::PhoneNumber::Home ) );
            break;

            case PBK_Number_Pager:
                a->insertPhoneNumber( KABC::PhoneNumber( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ), KABC::PhoneNumber::Pager ) );
            break;

            // using PhoneNumber::Voice for PBK_Number_Other
            case PBK_Number_Other:
                a->insertPhoneNumber( KABC::PhoneNumber( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ), KABC::PhoneNumber::Voice ) );
            break;

            case PBK_Text_Note:
                a->setNote( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_Email:
                a->insertEmail( (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_Text_Email2:
                a->insertEmail( (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_Text_URL:
                a->setUrl( KURL( (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) ) );
            break;

            case PBK_Text_Name:
                a->setFormattedName( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_LastName:
                a->setFamilyName( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_FirstName:
                a->setGivenName( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_Company:
                a->setOrganization( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_JobTitle:
                a->setTitle( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_StreetAddress:
                address.setStreet( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_City:
                address.setLocality( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_State:
                address.setRegion( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_Zip:
                address.setPostalCode( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_Country:
                address.setCountry( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_UserID:
                a->setUid( (char*)DecodeUnicodeConsole( Entry.Entries[x].Text ) );
            break;

            case PBK_Text_Custom1:
                a->insertCustom("KMobileTools", "custom_1", (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_Text_Custom2:
                a->insertCustom("KMobileTools", "custom_2", (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_Text_Custom3:
                a->insertCustom("KMobileTools", "custom_3", (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_Text_Custom4:
                a->insertCustom("KMobileTools", "custom_4", (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_Caller_Group:
                a->insertCustom("KMobileTools", "caller_grp", (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_RingtoneID:
                a->insertCustom("KMobileTools", "ringtone_id", (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_RingtoneFileSystemID:
                a->insertCustom("KMobileTools", "ringtone_fs_id", (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_SMSListID:
                a->insertCustom("KMobileTools", "sms_list_id", (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_PictureID:
                a->insertCustom("KMobileTools", "picture_id", (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

            case PBK_Text_Postal:
                a->insertCustom("KMobileTools", "text_postal", (char*)DecodeUnicodeConsole(Entry.Entries[x].Text) );
            break;

        }

    }

    if( !address.isEmpty() ) a->insertAddress( address );
    return a;

}

EditPbk::EditPbk( QValueList<KABC::Addressee>*abclist, Device *device, bool b_todelete, kmobiletoolsGammu_engine* parent, const char* name, GSM_MemoryType memtype ) : kmobiletoolsGammuJob(device, parent, name)
{
    p_abclist=abclist;
    p_oldAddressee=0;
    p_newAddressee=0;
    todelete=b_todelete;
    //TODO currently defaults to phone memory, make it configurable.
    p_memType = memtype;
}

EditPbk::EditPbk( KABC::Addressee *oldAddressee, KABC::Addressee *newAddressee, Device *device, kmobiletoolsGammu_engine *parent, const char *name)
    : kmobiletoolsGammuJob(device, parent, name)
{
    p_oldAddressee=oldAddressee;
    p_newAddressee=newAddressee;
    p_abclist=0;
}

void EditPbk::run()
{
    if( p_oldAddressee && p_newAddressee )
    {
        delEntry( p_oldAddressee );
        addEntry( p_newAddressee, static_cast<GSM_MemoryType> ( p_oldAddressee->custom("KMobileTools", "memslot").toInt() ), p_oldAddressee->custom("KMobileTools", "index").toInt() );
kdDebug() << "[ !!! ] Edit: " << p_oldAddressee->custom("KMobileTools", "memslot").toInt() << endl;
        return;
    }

    else if( !p_abclist->isEmpty() && !todelete )
    {
        for ( QValueListIterator<KABC::Addressee> it = p_abclist->begin(); it != p_abclist->end(); ++it )
        {
            KABC::Addressee* tmp = new KABC::Addressee(*it);
            addEntry( tmp, p_memType, engine->getFreeEntry( p_memType ) );
        }
    }

    else if( !p_abclist->isEmpty() && todelete )
    {
        for ( QValueListIterator<KABC::Addressee> it = p_abclist->begin(); it != p_abclist->end(); ++it )
        {
            KABC::Addressee* tmp=new KABC::Addressee(*it);
            delEntry( tmp );
        }
    }
}


void EditPbk::addEntry( KABC::Addressee *a, GSM_MemoryType type, int slot )
{
    GSM_MemoryEntry e = kabc2pbk( a );
    e.MemoryType = type;
    e.Location = slot;
    error = p_device->phone->SetMemory( &p_device->s, &e );
    kdDebug() << "[ !!! ] Adding new entry at location " << e.Location << ", for Memorytype " << e.MemoryType << ", Result: " << err.getErrorMsg(error) << endl;
}

void EditPbk::delEntry( KABC::Addressee *a )
{
    GSM_MemoryEntry e;

    e.MemoryType = static_cast<GSM_MemoryType> (a->custom("KMobileTools", "memslot").toInt());
    e.Location = a->custom("KMobileTools", "index").toInt();

    error = p_device->phone->DeleteMemory(&p_device->s, &e);
    kdDebug() << "[ !!! ] DeleteMemory " << err.getErrorMsg(error) << endl;
}

GSM_MemoryEntry EditPbk::kabc2pbk( KABC::Addressee *a )
{
    GSM_MemoryEntry m;
    QString tmp;
    m.EntriesNum = 1;

    if( !a->formattedName().isEmpty() )
    {
        kdDebug() << "[ !!! ] !a->formattedName().isEmpty()" << endl;
        EncodeUnicode( m.Entries[ m.EntriesNum ].Text, a->familyName().latin1(), a->familyName().length() );
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_Name;
        m.EntriesNum++;
    }

    if( !a->familyName().isEmpty() )
    {
        kdDebug() << "[ !!! ] !a->familyName().isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_LastName;
        EncodeUnicode( m.Entries[ m.EntriesNum ].Text, a->familyName().latin1(), a->familyName().length() );
        m.EntriesNum++;
    }

    if( !a->givenName().isEmpty() )
    {
        kdDebug() << "[ !!! ] !a->givenName().isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_FirstName;
        EncodeUnicode( m.Entries[ m.EntriesNum ].Text, a->givenName().latin1(), a->givenName().length() );
        m.EntriesNum++;
    }

    if( !a->categories().isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !a->categories().isEmpty()" << endl;
        QStringList list( a->categories() );

        for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {

            tmp = *it;
            m.Entries[ m.EntriesNum ].EntryType = PBK_Category;
            memcpy( m.Entries[ m.EntriesNum ].Text, tmp.latin1(), tmp.length() );
            m.EntriesNum++;
        }
    }

    if( !a->phoneNumbers().isEmpty() )
    {
        kdDebug() << "[ !!! ] !a->phoneNumbers().isEmpty()" << endl;
        QValueList<KABC::PhoneNumber> phonenumbers=a->phoneNumbers();
        QValueList<KABC::PhoneNumber>::iterator it;

        for ( it = phonenumbers.begin(); it != phonenumbers.end(); ++it )
        {
            switch( (*it).type() ) {

                case KABC::PhoneNumber::Work:
                    kdDebug() << "[ !!! ] Adding KABC::PhoneNumber::Work" << endl;
                    m.Entries[ m.EntriesNum ].EntryType = PBK_Number_Work;
                    EncodeUnicode( m.Entries[ m.EntriesNum ].Text, (*it).number().latin1(), (*it).number().length() );
                    m.EntriesNum++;
                break;

                case KABC::PhoneNumber::Home:
                    kdDebug() << "[ !!! ] Adding KABC::PhoneNumber::Home" << endl;
                    m.Entries[ m.EntriesNum ].EntryType = PBK_Number_Home;
                    EncodeUnicode( m.Entries[ m.EntriesNum ].Text, (*it).number().latin1(), (*it).number().length() );
                    m.EntriesNum++;
                break;

                case KABC::PhoneNumber::Pref:
                    kdDebug() << "[ !!! ] Adding KABC::PhoneNumber::Pref" << endl;
                    m.Entries[ m.EntriesNum ].EntryType = PBK_Number_Work;
                    EncodeUnicode( m.Entries[ m.EntriesNum ].Text, (*it).number().latin1(), (*it).number().length() );
                    m.EntriesNum++;
                break;

                case KABC::PhoneNumber::Cell:
                    kdDebug() << "[ !!! ] Adding KABC::PhoneNumber::Mobile" << endl;
                    m.Entries[ m.EntriesNum ].EntryType = PBK_Number_Mobile;
                    EncodeUnicode( m.Entries[ m.EntriesNum ].Text, (*it).number().latin1(), (*it).number().length() );
                    m.EntriesNum++;
                break;

                case KABC::PhoneNumber::Fax:
                    kdDebug() << "[ !!! ] Adding KABC::PhoneNumber::Fax" << endl;
                    m.Entries[ m.EntriesNum ].EntryType = PBK_Number_Fax;
                    EncodeUnicode( m.Entries[ m.EntriesNum ].Text, (*it).number().latin1(), (*it).number().length() );
                    m.EntriesNum++;
                break;

                case KABC::PhoneNumber::Pager:
                    kdDebug() << "[ !!! ] Adding KABC::PhoneNumber::Pager" << endl;
                    m.Entries[ m.EntriesNum ].EntryType = PBK_Number_Pager;
                    EncodeUnicode( m.Entries[ m.EntriesNum ].Text, (*it).number().latin1(), (*it).number().length() );
                    m.EntriesNum++;
                break;

            }

        }
    }

    if( !a->note().isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !a->note().isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_Note;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->note().latin1(), a->note().length() );
        m.EntriesNum++;
    }

    if( !a->url().isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !a->url().isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_URL;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->url().url().latin1(), a->url().url().length() );
        m.EntriesNum++;
    }

    if( !a->organization().isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !a->organization().isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_Company;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->organization().latin1(), a->organization().length() );
        m.EntriesNum++;
    }

    if( !a->title().isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !a->title().isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_JobTitle;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->title().latin1(), a->title().length() );
        m.EntriesNum++;
    }

    /* hmm uid never seems to be empty...
    if( !a->uid().isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !a->uid().isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_UserID;
        EncodeUnicode( m.Entries[ m.EntriesNum ].Text, (const unsigned char*)a->uid().latin1(), a->uid().length() );
        m.EntriesNum++;
    }*/

    if( !a->custom("KMobileTools", "custom_1" ).isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !custom_1.isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_Custom1;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->custom("KMobileTools", "custom_1" ).latin1(), a->custom("KMobileTools", "custom_1" ).length() );
        m.EntriesNum++;
    }

    if( !a->custom("KMobileTools", "custom_2" ).isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !custom_2.isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_Custom2;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->custom("KMobileTools", "custom_2" ).latin1(), a->custom("KMobileTools", "custom_2" ).length() );
        m.EntriesNum++;
    }

    if( !a->custom("KMobileTools", "custom_3" ).isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !custom_3.isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_Custom3;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->custom("KMobileTools", "custom_3" ).latin1(), a->custom("KMobileTools", "custom_3" ).length() );
        m.EntriesNum++;
    }

    if( !a->custom("KMobileTools", "custom_4" ).isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !custom_4.isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_Custom4;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->custom("KMobileTools", "custom_4" ).latin1(), a->custom("KMobileTools", "custom_4" ).length() );
        m.EntriesNum++;
    }

    if( !a->custom("KMobileTools", "caller_grp" ).isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !caller_grp.isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Caller_Group;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->custom("KMobileTools", "caller_grp" ).latin1(), a->custom("KMobileTools", "caller_grp" ).length() );
        m.EntriesNum++;
    }

    if( !a->custom("KMobileTools", "ringtone_id" ).isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !ringtone_id.isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_RingtoneID;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->custom("KMobileTools", "ringtone_id" ).latin1(), a->custom("KMobileTools", "ringtone_id" ).length() );
        m.EntriesNum++;
    }

    if( !a->custom("KMobileTools", "ringtone_fs_id" ).isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !ringtone_fs_id.isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_RingtoneFileSystemID;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->custom("KMobileTools", "ringtone_fs_id" ).latin1(), a->custom("KMobileTools", "ringtone_fs_id" ).length() );
        m.EntriesNum++;
    }

    if( !a->custom("KMobileTools", "sms_list_id" ).isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !sms_list_id.isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_SMSListID;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->custom("KMobileTools", "sms_list_id" ).latin1(), a->custom("KMobileTools", "sms_list_id" ).length() );
        m.EntriesNum++;
    }

    if( !a->custom("KMobileTools", "picture_id" ).isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !picture_id.isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_PictureID;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->custom("KMobileTools", "picture_id" ).latin1(), a->custom("KMobileTools", "picture_id" ).length() );
        m.EntriesNum++;
    }

    if( !a->custom("KMobileTools", "text_postal" ).isEmpty() ) 
    {
        kdDebug() << "[ !!! ] !text_postal.isEmpty()" << endl;
        m.Entries[ m.EntriesNum ].EntryType = PBK_Text_Postal;
        memcpy( m.Entries[ m.EntriesNum ].Text, a->custom("KMobileTools", "text_postal" ).latin1(), a->custom("KMobileTools", "text_postal" ).length() );
        m.EntriesNum++;
    }

    kdDebug() << "[ !!! ] Created MemoryEntry with " << m.EntriesNum << " Subentries." << endl;
    return m;
}
