/***************************************************************************
   Copyright (C) 2006
   by Marco Gulino <marco@kmobiletools.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include <qstring.h>
#include <kconfig.h>
#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>

#include "kmobiletools_devices.h"
#include "aboutdata.h"
using namespace KMobileTools;

QDict<KMobileTools::DevicesConfig> *KMobileTools::DevicesConfig::m_prefs = 0;

AboutData::AboutData()
    : KAboutData("kmobiletools", I18N_NOOP("KMobileTools"), KMOBILETOOLS_VERSION, I18N_NOOP("A KDE Mobile Phone Management Tool"),
                 License_GPL, I18N_NOOP("(C) 2004, 2005 KMobileTools developers"), 0,
                     "http://www.kmobiletools.org/")
{
    setVersion(KMOBILETOOLS_VERSION);
    addAuthor( "Marco Gulino", I18N_NOOP("Maintainer, core developer, AT engine and coordinator"), "marco@kmobiletools.org" );
    addAuthor( "Stefan Bogner", I18N_NOOP("Gammu engine"), "bochi@kmobiletools.org" );
    addCredit( "Alexander Rensmann", I18N_NOOP("AT engine, Siemens support"), "zerraxys@gmx.net" );
    addCredit( "Lee Olson", I18N_NOOP("KMobileTools main icon, images, artwork"), "clearbeast@gmail.com" );
}

AboutData::~AboutData()
{
}

KMobileTools::DevicesConfig *KMobileTools::DevicesConfig::prefs(const QString &groupName)
{
    if (!m_prefs)
    {
        m_prefs = new QDict<KMobileTools::DevicesConfig>();
        m_prefs->setAutoDelete(true);
    }
    KMobileTools::DevicesConfig *devicesPrefs = m_prefs->find(groupName);
    if (devicesPrefs)
    {
/*        devicesPrefs->readConfig();*/
        return devicesPrefs;
    }

    devicesPrefs = new KMobileTools::DevicesConfig(groupName);
    devicesPrefs->readConfig();
    m_prefs->insert(groupName, devicesPrefs);
    return devicesPrefs;
}

bool KMobileTools::DevicesConfig::hasPrefs(const QString &groupName)
{
    KConfig &config = *KGlobal::config();
    return config.hasGroup(groupName);
}

void KMobileTools::DevicesConfig::deletePrefs(const QString &groupName)
{
    KConfig &config = *KGlobal::config();
    config.deleteGroup(groupName);
    if (!m_prefs)
        return;
    m_prefs->remove(groupName);
}

const QString KMobileTools::DevicesConfig::deviceGroup( const QString &devicename)
{
    QString cgroup;
    for(int i=0; i<100; i++)
    {
        cgroup="device-%1";
        cgroup=cgroup.arg(i);
        if( KMobileTools::DevicesConfig::prefs( cgroup )->devicename() == devicename)
            return cgroup;
    }
    return NULL;
}

const QString KMobileTools::DevicesConfig::firstFreeGroup()
{
    QString cgroup;
    for(int i=0; i<100; i++)
    {
        cgroup="device-%1";
        cgroup=cgroup.arg(i);
        if( ! (KMobileTools::DevicesConfig::prefs( cgroup )->devicename().length() ) )
            return cgroup;
    }
    return NULL;
}
const QString KMobileTools::DevicesConfig::deviceType(const QString &groupName)
{
    switch ( KMobileTools::DevicesConfig::prefs(groupName)->devicetype() )
    {
        case 0:
            return i18n("AT engine");
        case 1:
            return i18n("Gammu engine");
        default:
            return i18n("Void engine");
    }
}

const QPixmap KMobileTools::DevicesConfig::deviceTypeIcon(const QString &groupName, KIcon::Group group, int size)
{
    switch ( KMobileTools::DevicesConfig::prefs(groupName)->devicetype() )
    {
        case 0:
            return KGlobal::iconLoader()->loadIcon("kmobiletools", group, size);
        case 1:
            return KGlobal::iconLoader()->loadIcon("blockdevice", group, size);
        default:
            return KGlobal::iconLoader()->loadIcon("connect_no", group, size);
    }
}

const QString KMobileTools::DevicesConfig::deviceTypeIconPath(const QString &groupName, int groupOrSize)
{
    switch ( KMobileTools::DevicesConfig::prefs(groupName)->devicetype() )
    {
        case 0:
            return KGlobal::iconLoader()->iconPath("kmobiletools", groupOrSize);
        case 1:
            return KGlobal::iconLoader()->iconPath("blockdevice", groupOrSize);
        default:
            return KGlobal::iconLoader()->iconPath("connect_no", groupOrSize);
    }
}
